@@ -13,6 +13,7 @@ Without lip height adjustment (sop_flute_3), flute is a semitone flat unless cor
 
 import profile, design
 
+from nesoni import config
 
 fingerings = {
     'pflute' : [
@@ -63,7 +64,7 @@ fingerings = {
         ('G5',   [0,0,0,1,1,1]),
         ('A5',   [0,0,0,0,1,1]),
         ('B5',   [0,0,0,0,0,1]),
-        ('C#6',  [0,0,0,0,0,0]),
+        ('C#6',  [1,1,1,0,0,0]),
         ('D6',   [1,1,1,1,1,0]),
 
         #('E6',   [0,1,1,1,1,1]),
@@ -111,6 +112,10 @@ fingerings = {
     ],
 }
 
+@config.Float_flag('embextra', 
+    'Constant controlling extra effective height of the embouchure hole due to lips, etc. '
+    'Small adjustments of this value will change the angle at which the flute needs to be blown '
+    'in order to be in tune.')
 class Flute_designer(design.Instrument_designer):
     closed_top = True
     
@@ -129,7 +134,14 @@ class Flute_designer(design.Instrument_designer):
     # 0.53 (sop_flute_8)   ~ a tiny biy low?, needed to push cork in 1.5mm
     #                        + perfect, printed plastic sop flute
     # 0.56                 ~ definitely high, printed plastic sop flute
-    hole_extra_height_by_diameter = [ 0.0 ] * 6 + [ 0.53 ]
+    
+    embextra = 0.53
+    
+    @property
+    def hole_extra_height_by_diameter(self):
+        return [ 0.0 ] * 6 + [ self.embextra ]
+    
+    #hole_extra_height_by_diameter = [ 0.0 ] * 6 + [ 0.53 ]
     
     initial_length = design.wavelength('D4') * 0.5
     
@@ -154,7 +166,11 @@ class Tapered_flute(Flute_designer):
 class Straight_flute(Flute_designer):
     inner_diameters = design.sqrt_scaler([ 18.4, 18.4, 21.0, 18.4, 18.4 ])
     initial_inner_fractions = [ 0.7, 0.8, 0.9 ]
-    min_inner_fraction_sep = [ 0.0, 0.03, 0.0, 0.0 ]
+    
+    min_inner_fraction_sep = [ 0.5, 0.03, 0.0, 0.0 ]
+    # Note constraint of bulge to upper half of tube.
+    # There seems to be an alternate solution for the folk flute
+    #   where it's stretched out over 3/4 of the flute's length.
 
     outer_diameters = design.sqrt_scaler([ 28.0, 28.0 ])
 
@@ -168,17 +184,31 @@ class Pflute(Flute_designer):
     balance = [ 0.05, None, None, 0.05 ]    
     hole_angles = [ -30.0, -30.0, 30.0, -30.0, 30.0, -30.0, 0.0 ]
 
+@config.help("""\
+Design a flute with a tapered bore and recorder-like fingering system.
+""")
 class Design_tapered_pflute(Tapered_flute, Pflute): pass
+
+@config.help("""\
+Design a flute with a straight bore and recorder-like fingering system.
+""")
 class Design_straight_pflute(Straight_flute, Pflute): pass
 
 class Folk_flute(Flute_designer):
     fingering_system = 'folk'
-    balance = [ 0.025, 0.1, 0.1, 0.025 ]    
-    hole_angles = [ -30.0, 30.0, 0.0,  -30.0, 0.0, 30.0, 0.0 ]
+    balance = [ 0.05, None, None, 0.05 ]    
+    hole_angles = [ -30.0, 30.0, 0.0,  0.0, 0.0, 0.0, 0.0 ]
     min_hole_diameters = design.sqrt_scaler([ 7.5 ] * 6  + [ 12.2 ])
     max_hole_diameters = design.sqrt_scaler([ 11.4 ] * 6 + [ 13.9 ])
 
+@config.help("""\
+Design a flute with a tapered bore and pennywhistle-like fingering system.
+""")
 class Design_tapered_folk_flute(Tapered_flute, Folk_flute): pass
+
+@config.help("""\
+Design a flute with a straight bore and pennywhistle-like fingering system.
+""")
 class Design_straight_folk_flute(Straight_flute, Folk_flute): pass
 
 
